import { defineStore } from 'pinia'
import core from '@/core'

export const useDocumentStore = defineStore({
  id: 'document',
  state: () => ({
    totalPages: 0,
    loadingProgress: 0,
    annotationsCount: 0,
    info: null,
    measure: {
      precision: 1,
      scale: {
        origin: {
          num: 1,
          unit: 'pt'
        },
        actual: {
          num: 1,
          unit: 'pt'
        }
      },
      distance: 0
    },
    optionUrl: null,
    markupTool: 'highlight',
    highlight: {
      color: '#FFEC66'
    },
    underline: {
      color: '#FF0000'
    },
    squiggly: {
      color: '#FF0000'
    },
    strikethrough: {
      color: '#FF0000'
    },
    shapeTool: 'square',
    square: {
      color: '#FF0000',
      opacity: '0.8',
      stroke: '1pt'
    },
    circle: {
      color: '#FF0000',
      opacity: '0.8',
      stroke: '1pt'
    },
    arrow: {
      color: '#FF0000',
      opacity: '0.8',
      stroke: '1pt'
    },
    line: {
      color: '#FF0000',
      opacity: '0.8',
      stroke: '1pt'
    },
    ink: {
      color: '#FF0000',
      opacity: '0.8',
      stroke: '2pt'
    },
    freetext: {
      color: '#000000'
    },
    text: {
      color: '#000000'
    },
    activeTool: '',
    annotations: null,
  }),
  getters: {
    getTotalPages () {
      return this.totalPages
    },
    getLoadingProgress () {
      return this.loadingProgress
    },
    getAnnotationsCount () {
      return this.annotationsCount + ' annotation' + (this.annotationsCount > 1 ? 's' : '')
    },
    getMeasure () {
      return this.measure
    },
    getPrecision () {
      return this.measure.precision === 1 ? 0.1 : 0.01
    },
    getInfo () {
      return this.info
    },
    getOptionUrl () {
      return this.optionUrl
    },
    getMarkupToolState () {
      return this.markupTool
    },
    getLineType () {
      return this.line.type
    },
    getActiveToolColor () {
      if (this.activeTool) {
        return this[this.activeTool].color
      }
    },
    getActiveTool () {
      return this.activeTool
    },
    getShapeToolState () {
      return this.shapeTool
    },
    getAllAnnotations () {
      return this.annotations
    },
    getAnnotations (pageNum) {
      return this.annotations && this.annotations[pageNum]
    },
    getPagesAnnotCount (state) {
      let count = 0
      const keys = Object.keys(this.annotations)
      for (let key of keys) {
        count += this.annotations[key][0].pageAnnotCount
      }
      return count
    }
  },
  actions: {
    resetSetting () {
      this.totalPages = 0
      this.loadingProgress = 0
      this.annotationsCount = 0
    },
    setTotalPages (totalPages) {
      this.totalPages = totalPages
    },
    setLoadingProgress (percent) {
      this.loadingProgress = percent
    },
    setAnnotationsCount (count) {
      this.annotationsCount = count
    },
    setOptionUrl (optionUrl) {
      this.optionUrl = optionUrl
    },
    setToken (info) {
      this.info = info
    },
    setDistance (distance) {
      this.measure.distance = Math.round(distance * 10) / 10
    },
    setMarkupToolState (tool) {
      this.markupTool = tool
    },
    setMarkupToolColor (color) {
      this[this.markupTool] = color
    },
    setShapeToolState (tool) {
      this.shapeTool = tool
    },
    setLineType (type) {
      this.line.type = type
    },
    setLineColor (color) {
      this.line.color = color
    },
    setActiveTool (tool) {
      this.activeTool = tool
    },
    setActiveToolColor (color) {
      if (this.activeTool) {
        this[this.activeTool].color = color
        core.setTool({
          tool: this.activeTool,
          color: color
        })
      }
    },
    setToolState (tool) {
      if (this.activeTool === tool || !tool) {
        this.activeTool = ''
        core.setTool({
          tool: '',
          color: ''
        })
      } else {
        this.activeTool = tool
        core.setTool({
          tool,
          color: this[tool].color
        })
      }
    },
    initAnnotations (annotations) {
      this.annotations = null
      this.annotations = annotations
    },
    setAnnotations (annotation, type) {
      const annotations = this.annotations
      if (type === 'add') {
        if (!annotations[annotation[page]]) {
          annotations[page] = []
        }
        annotations[page].push(annotation)
      } else {
        const annotationsPage = annotations[annotation[page]]
        const deleteIndex = annotationsPage.findIndex((item) => item.index ===  annotation.index)
        if (type === 'delete') {
          annotationsPage[deleteIndex].isDelete = true
        } else {
          annotationsPage[deleteIndex] = {
            ...annotationsPage[deleteIndex],
            ...annotation
          }
        }
      }
    }
  }
})
